@@ -98,7 +98,9 @@ void do_ncheck(int argc, char **argv)
 		goto error_out;
 	}
 
-	retval = ext2fs_get_next_inode(scan, &ino, &inode);
+	do {
+		retval = ext2fs_get_next_inode(scan, &ino, &inode);
+	} while (retval == EXT2_ET_BAD_BLOCK_IN_INODE_TABLE);
 	if (retval) {
 		com_err("ncheck", retval, "while starting inode scan");
 		goto error_out;
@@ -132,7 +134,10 @@ void do_ncheck(int argc, char **argv)
 			break;
 
 	next:
-		retval = ext2fs_get_next_inode(scan, &ino, &inode);
+		do {
+			retval = ext2fs_get_next_inode(scan, &ino, &inode);
+		} while (retval == EXT2_ET_BAD_BLOCK_IN_INODE_TABLE);
+
 		if (retval) {
 			com_err("ncheck", retval,
 				"while doing inode scan");
