@@ -20,6 +20,7 @@
  * 	%dn	<dirent>->name		string
  * 	%dr	<dirent>->rec_len
  * 	%dl	<dirent>->name_len
+ * 	%dt	<dirent>->filetype
  * 	%D	<dir> 			inode number
  * 	%g	<group>			integer
  * 	%i	<ino>			inode number
@@ -134,7 +135,7 @@ static const char *special_inode_name[] =
  * This function does "safe" printing.  It will convert non-printable
  * ASCII characters using '^' and M- notation.
  */
-static void safe_print(const unsigned char *cp, int len)
+static void safe_print(const char *cp, int len)
 {
 	unsigned char	ch;
 
@@ -301,6 +302,9 @@ static _INLINE_ void expand_dirent_expression(char ch,
 	case 'l':
 		printf("%u", dirent->name_len & 0xFF);
 		break;
+	case 't':
+		printf("%u", dirent->name_len >> 8);
+		break;
 	default:
 	no_dirent:
 		printf("%%D%c", ch);
