@@ -1,7 +1,7 @@
 /*
  * pass3.c -- pass #3 of e2fsck: Check for directory connectivity
  *
- * Copyright (C) 1993, 1994, 1995, 1996, 1997 Theodore Ts'o.
+ * Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999 Theodore Ts'o.
  *
  * %Begin-Header%
  * This file may be redistributed under the terms of the GNU Public
@@ -451,7 +451,7 @@ ino_t get_lost_and_found(e2fsck_t ctx)
 	/*
 	 * Finally, create the directory link
 	 */
-	pctx.errcode = ext2fs_link(fs, EXT2_ROOT_INO, name, ino, 0);
+	pctx.errcode = ext2fs_link(fs, EXT2_ROOT_INO, name, ino, EXT2_FT_DIR);
 	if (pctx.errcode) {
 		pctx.str = "ext2fs_link";
 		fix_problem(ctx, PR_3_CREATE_LPF_ERROR, &pctx);
@@ -474,15 +474,17 @@ ino_t get_lost_and_found(e2fsck_t ctx)
 /*
  * This routine will connect a file to lost+found
  */
-int e2fsck_reconnect_file(e2fsck_t ctx, ino_t inode)
+int e2fsck_reconnect_file(e2fsck_t ctx, ino_t ino)
 {
 	ext2_filsys fs = ctx->fs;
 	errcode_t	retval;
 	char		name[80];
 	struct problem_context	pctx;
+	struct ext2_inode 	inode;
+	int		file_type = 0;
 
 	clear_problem_context(&pctx);
-	pctx.ino = inode;
+	pctx.ino = ino;
 
 	if (!bad_lost_and_found && !lost_and_found) {
 		lost_and_found = get_lost_and_found(ctx);
@@ -494,8 +496,10 @@ int e2fsck_reconnect_file(e2fsck_t ctx, ino_t inode)
 		return 1;
 	}
 	
-	sprintf(name, "#%lu", inode);
-	retval = ext2fs_link(fs, lost_and_found, name, inode, 0);
+	sprintf(name, "#%lu", ino);
+	if (ext2fs_read_inode(fs, ino, &inode) == 0)
+		file_type = ext2_file_type(inode.i_mode);
+	retval = ext2fs_link(fs, lost_and_found, name, ino, file_type);
 	if (retval == EXT2_ET_DIR_NO_SPACE) {
 		if (!fix_problem(ctx, PR_3_EXPAND_LF_DIR, &pctx))
 			return 1;
@@ -505,14 +509,14 @@ int e2fsck_reconnect_file(e2fsck_t ctx, ino_t inode)
 			fix_problem(ctx, PR_3_CANT_EXPAND_LPF, &pctx);
 			return 1;
 		}
-		retval = ext2fs_link(fs, lost_and_found, name, inode, 0);
+		retval = ext2fs_link(fs, lost_and_found, name, ino, file_type);
 	}
 	if (retval) {
 		pctx.errcode = retval;
 		fix_problem(ctx, PR_3_CANT_RECONNECT, &pctx);
 		return 1;
 	}
-	adjust_inode_count(ctx, inode, +1);
+	adjust_inode_count(ctx, ino, +1);
 
 	return 0;
 }
@@ -538,16 +542,25 @@ static errcode_t adjust_inode_count(e2fsck_t ctx, ino_t ino, int adj)
 	       inode.i_links_count);
 #endif
 
-	inode.i_links_count += adj;
 	if (adj == 1) {
 		ext2fs_icount_increment(ctx->inode_count, ino, 0);
+		if (inode.i_links_count == (__u16) ~0)
+			return 0;
 		ext2fs_icount_increment(ctx->inode_link_info, ino, 0);
-	} else {
+		inode.i_links_count++;
+	} else if (adj == -1) {
 		ext2fs_icount_decrement(ctx->inode_count, ino, 0);
+		if (inode.i_links_count == 0)
+			return 0;
 		ext2fs_icount_decrement(ctx->inode_link_info, ino, 0);
+		inode.i_links_count--;
+	} else {
+		/* Should never happen */
+		printf("Debug error in e2fsck adjust_inode_count, "
+		       "should never happen.\n");
+		exit(1);
 	}
 	
-
 	retval = ext2fs_write_inode(fs, ino, &inode);
 	if (retval)
 		return retval;
@@ -635,9 +648,10 @@ static void fix_dotdot(e2fsck_t ctx, struct dir_info *dir, ino_t parent)
  */
 
 struct expand_dir_struct {
-	int	done;
-	errcode_t	err;
-	e2fsck_t	ctx;
+	int			done;
+	int			newblocks;
+	errcode_t		err;
+	e2fsck_t		ctx;
 };
 
 static int expand_dir_proc(ext2_filsys fs,
@@ -690,6 +704,8 @@ static int expand_dir_proc(ext2_filsys fs,
 	ext2fs_mark_block_bitmap(ctx->block_found_map, new_blk);
 	ext2fs_mark_block_bitmap(fs->block_map, new_blk);
 	ext2fs_mark_bb_dirty(fs);
+	es->newblocks++;
+	
 	if (es->done)
 		return (BLOCK_CHANGED | BLOCK_ABORT);
 	else
@@ -711,13 +727,14 @@ static errcode_t expand_directory(e2fsck_t ctx, ino_t dir)
 	 * them.
 	 */
 	e2fsck_read_bitmaps(ctx);
-	
+
 	retval = ext2fs_check_directory(fs, dir);
 	if (retval)
 		return retval;
 	
 	es.done = 0;
 	es.err = 0;
+	es.newblocks = 0;
 	es.ctx = ctx;
 	
 	retval = ext2fs_block_iterate(fs, dir, BLOCK_FLAG_APPEND,
@@ -736,7 +753,7 @@ static errcode_t expand_directory(e2fsck_t ctx, ino_t dir)
 		return retval;
 	
 	inode.i_size += fs->blocksize;
-	inode.i_blocks += fs->blocksize / 512;
+	inode.i_blocks += (fs->blocksize / 512) * es.newblocks;
 
 	e2fsck_write_inode(ctx, dir, &inode, "expand_directory");
 
