@@ -413,6 +413,16 @@ static const struct e2fsck_problem problem_table[] = {
 	  "@f contains large files, but lacks LARGE_FILE flag in @S.\n",
 	  PROMPT_FIX, 0 },
 	  
+	/* Imagic flag set on an inode when filesystem doesn't support it */
+	{ PR_1_SET_IMAGIC,
+	  "@i %i has imagic flag set.  ",
+	  PROMPT_CLEAR, 0 },
+
+	/* Immutable flag set on a device or socket inode */
+	{ PR_1_SET_IMMUTABLE,
+	  "Special (device/socket/fifo) @i %i has immutable flag set.  ",
+	  PROMPT_CLEAR, PR_PREEN_OK | PR_PREEN_NO | PR_NO_OK },
+
 	/* Pass 1b errors */
 
 	/* Pass 1B: Rescan for duplicate/bad blocks */
@@ -693,6 +703,26 @@ static const struct e2fsck_problem problem_table[] = {
 	  "@i %i (%Q) is an @I socket.\n",
 	  PROMPT_CLEAR, 0 },
 
+	/* Directory filetype not set */
+	{ PR_2_SET_FILETYPE,
+	  "Setting filetype for @E to %N.\n",
+	  PROMPT_NONE, PR_PREEN_OK | PR_NO_OK | PR_NO_NOMSG },
+
+	/* Directory filetype incorrect */
+	{ PR_2_BAD_FILETYPE,
+	  "@E has an incorrect filetype (was %dt, should be %N)\n",
+	  PROMPT_FIX, 0 },
+
+	/* Directory filetype set on filesystem */
+	{ PR_2_CLEAR_FILETYPE,
+	  "@E has filetype set\n",
+	  PROMPT_CLEAR, PR_PREEN_OK },
+
+	/* Directory filename is null */
+	{ PR_2_NULL_NAME,
+	  "@E has a zero-length name\n",
+	  PROMPT_CLEAR, 0 },
+
 	/* Pass 3 errors */
 
 	/* Pass 3: Checking directory connectivity */
@@ -1038,7 +1068,11 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 		printf("Unhandled error code (%d)!\n", code);
 		return 0;
 	}
-	def_yn = (ptr->flags & PR_NO_DEFAULT) ? 0 : 1;
+	def_yn = 1;
+	if ((ptr->flags & PR_NO_DEFAULT) ||
+	    ((ptr->flags & PR_PREEN_NO) && (ctx->options & E2F_OPT_PREEN)) ||
+	    (ctx->options & E2F_OPT_NO))
+		def_yn= 0;
 
 	/*
 	 * Do special latch processing.  This is where we ask the
@@ -1060,6 +1094,9 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 	if ((ptr->flags & PR_PREEN_NOMSG) &&
 	    (ctx->options & E2F_OPT_PREEN))
 		suppress++;
+	if ((ptr->flags & PR_NO_NOMSG) &&
+	    (ctx->options & E2F_OPT_NO))
+		suppress++;
 	if (!suppress) {
 		message = ptr->e2p_description;
 		if (ctx->options & E2F_OPT_PREEN) {
