@@ -236,6 +236,12 @@ struct problem_context {
 /* Filesystem contains large files, but has no such flag in sb */
 #define PR_1_FEATURE_LARGE_FILES 	0x01002E
 
+/* Imagic flag set on an inode when filesystem doesn't support it */
+#define PR_1_SET_IMAGIC			0x01002F
+
+/* Immutable flag set on a device or socket inode */
+#define PR_1_SET_IMMUTABLE		0x010030
+	
 /*
  * Pass 1b errors
  */
@@ -403,7 +409,7 @@ struct problem_context {
 #define PR_2_DEALLOC_INODE	0x020023
 
 /* Directory entry for '.' is big.  Split? */
-#define PR_2_SPLIT_DOT		0x0200024
+#define PR_2_SPLIT_DOT		0x020024
 
 /* Illegal FIFO */
 #define PR_2_BAD_FIFO		0x020025
@@ -411,6 +417,18 @@ struct problem_context {
 /* Illegal socket */
 #define PR_2_BAD_SOCKET		0x020026
 
+/* Directory filetype not set */
+#define PR_2_SET_FILETYPE	0x020027
+
+/* Directory filetype incorrect */
+#define PR_2_BAD_FILETYPE	0x020028
+
+/* Directory filetype set when it shouldn't be */
+#define PR_2_CLEAR_FILETYPE	0x020029
+
+/* Directory filename can't be zero-length  */
+#define PR_2_NULL_NAME		0x020030
+
 /*
  * Pass 3 errors
  */
