@@ -13,7 +13,19 @@
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
+
+#ifdef HAVE_CONIO_H
+#undef HAVE_TERMIOS_H
+#include <conio.h>
+#define read_a_char()	getch()
+#else
+#ifdef HAVE_TERMIOS_H
 #include <termios.h>
+#endif
+#include <stdio.h>
+#define read_a_char()	getchar()
+#endif
+
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -54,15 +66,18 @@ void *e2fsck_allocate_memory(e2fsck_t ctx, unsigned int size,
 int ask_yn(const char * string, int def)
 {
 	int		c;
-	struct termios	termios, tmp;
 	const char	*defstr;
 
+#ifdef HAVE_TERMIOS_H
+	struct termios	termios, tmp;
+
 	tcgetattr (0, &termios);
 	tmp = termios;
 	tmp.c_lflag &= ~(ICANON | ECHO);
 	tmp.c_cc[VMIN] = 1;
 	tmp.c_cc[VTIME] = 0;
 	tcsetattr (0, TCSANOW, &tmp);
+#endif
 
 	if (def == 1)
 		defstr = "<y>";
@@ -73,7 +88,7 @@ int ask_yn(const char * string, int def)
 	printf("%s%s? ", string, defstr);
 	while (1) {
 		fflush (stdout);
-		if ((c = getchar()) == EOF)
+		if ((c = read_a_char()) == EOF)
 			break;
 		c = toupper(c);
 		if (c == 'Y') {
@@ -91,7 +106,9 @@ int ask_yn(const char * string, int def)
 		printf ("yes\n\n");
 	else
 		printf ("no\n\n");
+#ifdef HAVE_TERMIOS_H
 	tcsetattr (0, TCSANOW, &termios);
+#endif
 	return def;
 }
 
@@ -297,3 +314,31 @@ blk_t get_backup_sb(ext2_filsys fs)
 	return fs->super->s_blocks_per_group + 1;
 }
 
+/*
+ * Given a mode, return the ext2 file type
+ */
+int ext2_file_type(unsigned int mode)
+{
+	if (LINUX_S_ISREG(mode))
+		return EXT2_FT_REG_FILE;
+
+	if (LINUX_S_ISDIR(mode))
+		return EXT2_FT_DIR;
+	
+	if (LINUX_S_ISCHR(mode))
+		return EXT2_FT_CHRDEV;
+	
+	if (LINUX_S_ISBLK(mode))
+		return EXT2_FT_BLKDEV;
+	
+	if (LINUX_S_ISLNK(mode))
+		return EXT2_FT_SYMLINK;
+
+	if (LINUX_S_ISFIFO(mode))
+		return EXT2_FT_FIFO;
+	
+	if (LINUX_S_ISSOCK(mode))
+		return EXT2_FT_SOCK;
+	
+	return 0;
+}
