@@ -1,46 +1,34 @@
 /*
  * Copyright 1987, 1988 by MIT Student Information Processing Board.
- *
- * For copyright info, see mit-sipb-copyright.h.
+ * 
+ * Permission to use, copy, modify, and distribute this software and
+ * its documentation for any purpose is hereby granted, provided that
+ * the names of M.I.T. and the M.I.T. S.I.P.B. not be used in
+ * advertising or publicity pertaining to distribution of the software
+ * without specific, written prior permission.  M.I.T. and the
+ * M.I.T. S.I.P.B. make no representations about the suitability of
+ * this software for any purpose.  It is provided "as is" without
+ * express or implied warranty.
  */
 
 #include <stdio.h>
-#include "mit-sipb-copyright.h"
+#include "com_err.h"
+#include "error_table.h"
+#include "internal.h"
 
-#if __STDC__ || defined(STDARG_PROTOTYPES)
-#include <stdarg.h>
-#else
+#if !defined(__STDC__) && !defined(STDARG_PROTOTYPES)
 #include <varargs.h>
 #define VARARGS
 #endif
 
-#include "error_table.h"
-#include "internal.h"
-
-#ifdef notdef
-/*
- * Protect us from header version (externally visible) of com_err, so
- * we can survive in a <varargs.h> environment.  I think.
- */
-#define com_err com_err_external
-#include "com_err.h"
-#undef com_err
-#else
-#include "com_err.h"
-#endif
-
-#if ! lint
-static const char rcsid[] =
-    "$Header: /mit/krb5/.cvsroot/src/util/et/com_err.c,v 5.1 1993/06/03 12:25:24 tytso Exp $";
-#endif	/* ! lint */
-
 static void
 #ifdef __STDC__
-    default_com_err_proc (const char *whoami, long code, const char *fmt, va_list args)
+    default_com_err_proc (const char *whoami, errcode_t code, const
+			  char *fmt, va_list args)
 #else
     default_com_err_proc (whoami, code, fmt, args)
     const char *whoami;
-    long code;
+    errcode_t code;
     const char *fmt;
     va_list args;
 #endif
@@ -56,32 +44,37 @@ static void
     if (fmt) {
         vfprintf (stderr, fmt, args);
     }
-    putc('\n', stderr);
     /* should do this only on a tty in raw mode */
     putc('\r', stderr);
+    putc('\n', stderr);
     fflush(stderr);
 }
 
 #ifdef __STDC__
-typedef void (*errf) (const char *, long, const char *, va_list);
+typedef void (*errf) (const char *, errcode_t, const char *, va_list);
 #else
 typedef void (*errf) ();
 #endif
 
 errf com_err_hook = default_com_err_proc;
 
+#ifdef __STDC__
+void com_err_va (const char *whoami, errcode_t code, const char *fmt,
+		 va_list args)
+#else
 void com_err_va (whoami, code, fmt, args)
     const char *whoami;
-    long code;
+    errcode_t code;
     const char *fmt;
     va_list args;
+#endif
 {
     (*com_err_hook) (whoami, code, fmt, args);
 }
 
 #ifndef VARARGS
 void com_err (const char *whoami,
-	      long code,
+	      errcode_t code,
 	      const char *fmt, ...)
 {
 #else
@@ -89,7 +82,7 @@ void com_err (va_alist)
     va_dcl
 {
     const char *whoami, *fmt;
-    long code;
+    errcode_t code;
 #endif
     va_list pvar;
 
@@ -98,7 +91,7 @@ void com_err (va_alist)
 #ifdef VARARGS
     va_start (pvar);
     whoami = va_arg (pvar, const char *);
-    code = va_arg (pvar, long);
+    code = va_arg (pvar, errcode_t);
     fmt = va_arg (pvar, const char *);
 #else
     va_start(pvar, fmt);
