@@ -15,17 +15,23 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <fcntl.h>
 #include <time.h>
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -40,11 +46,14 @@ struct set_badblock_record {
 	errcode_t	err;
 };
 
-static int set_bad_block_proc(ext2_filsys fs, blk_t *block_nr, int blockcnt,
-			      blk_t ref_block, int ref_offset, void *private);
-static int clear_bad_block_proc(ext2_filsys fs, blk_t *block_nr, int blockcnt,
+static int set_bad_block_proc(ext2_filsys fs, blk_t *block_nr,
+			      e2_blkcnt_t blockcnt,
+			      blk_t ref_block, int ref_offset,
+			      void *priv_data);
+static int clear_bad_block_proc(ext2_filsys fs, blk_t *block_nr,
+				e2_blkcnt_t blockcnt,
 				blk_t ref_block, int ref_offset,
-				void *private);
+				void *priv_data);
 	
 /*
  * Given a bad blocks bitmap, update the bad blocks inode to reflect
@@ -65,15 +74,14 @@ errcode_t ext2fs_update_bb_inode(ext2_filsys fs, ext2_badblocks_list bb_list)
 	rec.bad_block_count = 0;
 	rec.ind_blocks_size = rec.ind_blocks_ptr = 0;
 	rec.max_ind_blocks = 10;
-	rec.ind_blocks = malloc(rec.max_ind_blocks * sizeof(blk_t));
-	if (!rec.ind_blocks)
-		return ENOMEM;
+	retval = ext2fs_get_mem(rec.max_ind_blocks * sizeof(blk_t),
+				(void **) &rec.ind_blocks);
+	if (retval)
+		return retval;
 	memset(rec.ind_blocks, 0, rec.max_ind_blocks * sizeof(blk_t));
-	rec.block_buf = malloc(fs->blocksize);
-	if (!rec.block_buf) {
-		retval = ENOMEM;
+	retval = ext2fs_get_mem(fs->blocksize, (void **) &rec.block_buf);
+	if (retval)
 		goto cleanup;
-	}
 	memset(rec.block_buf, 0, fs->blocksize);
 	rec.err = 0;
 	
@@ -143,8 +151,8 @@ errcode_t ext2fs_update_bb_inode(ext2_filsys fs, ext2_badblocks_list bb_list)
 		goto cleanup;
 	
 cleanup:
-	free(rec.ind_blocks);
-	free(rec.block_buf);
+	ext2fs_free_mem((void **) &rec.ind_blocks);
+	ext2fs_free_mem((void **) &rec.block_buf);
 	return retval;
 }
 
@@ -154,12 +162,19 @@ cleanup:
  * Clear the bad blocks in the bad block inode, while saving the
  * indirect blocks.
  */
-static int clear_bad_block_proc(ext2_filsys fs, blk_t *block_nr, int blockcnt,
-				blk_t ref_block, int ref_offset, void *private)
+#ifdef __TURBOC__
+#pragma argsused
+#endif
+static int clear_bad_block_proc(ext2_filsys fs, blk_t *block_nr,
+				e2_blkcnt_t blockcnt,
+				blk_t ref_block, int ref_offset,
+				void *priv_data)
 {
 	struct set_badblock_record *rec = (struct set_badblock_record *)
-		private;
-	int	group;
+		priv_data;
+	errcode_t	retval;
+	int		group;
+	unsigned long 	old_size;
 
 	if (!*block_nr)
 		return 0;
@@ -175,12 +190,14 @@ static int clear_bad_block_proc(ext2_filsys fs, blk_t *block_nr, int blockcnt,
 
 	if (blockcnt < 0) {
 		if (rec->ind_blocks_size >= rec->max_ind_blocks) {
+			old_size = rec->max_ind_blocks * sizeof(blk_t);
 			rec->max_ind_blocks += 10;
-			rec->ind_blocks = realloc(rec->ind_blocks,
-						  rec->max_ind_blocks *
-						  sizeof(blk_t));
-			if (!rec->ind_blocks) {
-				rec->err = ENOMEM;
+			retval = ext2fs_resize_mem(old_size, 
+				   rec->max_ind_blocks * sizeof(blk_t),
+				   (void **) &rec->ind_blocks);
+			if (retval) {
+				rec->max_ind_blocks -= 10;
+				rec->err = retval;
 				return BLOCK_ABORT;
 			}
 		}
@@ -207,12 +224,15 @@ static int clear_bad_block_proc(ext2_filsys fs, blk_t *block_nr, int blockcnt,
  *
  * Set the block list in the bad block inode, using the supplied bitmap.
  */
+#ifdef __TURBOC__
+#pragma argsused
+#endif
 static int set_bad_block_proc(ext2_filsys fs, blk_t *block_nr,
-			      int blockcnt, blk_t ref_block, 
-			      int ref_offset, void *private)
+			      e2_blkcnt_t blockcnt, blk_t ref_block, 
+			      int ref_offset, void *priv_data)
 {
 	struct set_badblock_record *rec = (struct set_badblock_record *)
-		private;
+		priv_data;
 	errcode_t	retval;
 	blk_t		blk;
 	int		group;
