@@ -12,17 +12,23 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 #include <fcntl.h>
 #include <time.h>
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -30,12 +36,14 @@ static errcode_t make_bitmap(__u32 start, __u32 end, __u32 real_end,
 			     const char *descr, char *init_map,
 			     ext2fs_generic_bitmap *ret)
 {
-	ext2fs_generic_bitmap bitmap;
-	int	size;
+	ext2fs_generic_bitmap	bitmap;
+	errcode_t		retval;
+	size_t			size;
 
-	bitmap = malloc(sizeof(struct ext2fs_struct_generic_bitmap));
-	if (!bitmap)
-		return ENOMEM;
+	retval = ext2fs_get_mem(sizeof(struct ext2fs_struct_generic_bitmap),
+				(void **) &bitmap);
+	if (retval)
+		return retval;
 
 	bitmap->magic = EXT2_ET_MAGIC_GENERIC_BITMAP;
 	bitmap->fs = NULL;
@@ -44,21 +52,22 @@ static errcode_t make_bitmap(__u32 start, __u32 end, __u32 real_end,
 	bitmap->real_end = real_end;
 	bitmap->base_error_code = EXT2_ET_BAD_GENERIC_MARK;
 	if (descr) {
-		bitmap->description = malloc(strlen(descr)+1);
-		if (!bitmap->description) {
-			free(bitmap);
-			return ENOMEM;
+		retval = ext2fs_get_mem(strlen(descr)+1,
+					(void **) &bitmap->description);
+		if (retval) {
+			ext2fs_free_mem((void **) &bitmap);
+			return retval;
 		}
 		strcpy(bitmap->description, descr);
 	} else
 		bitmap->description = 0;
 
-	size = ((bitmap->real_end - bitmap->start) / 8) + 1;
-	bitmap->bitmap = malloc(size);
-	if (!bitmap->bitmap) {
-		free(bitmap->description);
-		free(bitmap);
-		return ENOMEM;
+	size = (size_t) (((bitmap->real_end - bitmap->start) / 8) + 1);
+	retval = ext2fs_get_mem(size, (void **) &bitmap->bitmap);
+	if (retval) {
+		ext2fs_free_mem((void **) &bitmap->description);
+		ext2fs_free_mem((void **) &bitmap);
+		return retval;
 	}
 
 	if (init_map)
@@ -82,18 +91,28 @@ errcode_t ext2fs_copy_bitmap(ext2fs_generic_bitmap src,
 			     ext2fs_generic_bitmap *dest)
 {
 	errcode_t		retval;
-	ext2fs_generic_bitmap	new;
+	ext2fs_generic_bitmap	new_map;
 
 	retval = make_bitmap(src->start, src->end, src->real_end,
-			     src->description, src->bitmap, &new);
+			     src->description, src->bitmap, &new_map);
 	if (retval)
 		return retval;
-	new->magic = src->magic;
-	new->fs = src->fs;
-	new->base_error_code = src->base_error_code;
+	new_map->magic = src->magic;
+	new_map->fs = src->fs;
+	new_map->base_error_code = src->base_error_code;
+	*dest = new_map;
 	return 0;
 }
 
+void ext2fs_set_bitmap_padding(ext2fs_generic_bitmap map)
+{
+	__u32	i, j;
+
+	for (i=map->end+1, j = i - map->start; i <= map->real_end; i++, j++)
+		ext2fs_set_bit(j, map->bitmap);
+
+	return;
+}	
 
 errcode_t ext2fs_allocate_inode_bitmap(ext2_filsys fs,
 				       const char *descr,
@@ -186,7 +205,7 @@ void ext2fs_clear_inode_bitmap(ext2fs_inode_bitmap bitmap)
 		return;
 
 	memset(bitmap->bitmap, 0,
-	       ((bitmap->real_end - bitmap->start) / 8) + 1);
+	       (size_t) (((bitmap->real_end - bitmap->start) / 8) + 1));
 }
 
 void ext2fs_clear_block_bitmap(ext2fs_block_bitmap bitmap)
@@ -195,5 +214,5 @@ void ext2fs_clear_block_bitmap(ext2fs_block_bitmap bitmap)
 		return;
 
 	memset(bitmap->bitmap, 0,
-	       ((bitmap->real_end - bitmap->start) / 8) + 1);
+	       (size_t) (((bitmap->real_end - bitmap->start) / 8) + 1));
 }
