@@ -11,26 +11,34 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
+#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fsP.h"
 
 struct ext2_struct_inode_scan {
-	int			magic;
+	errcode_t		magic;
 	ext2_filsys		fs;
 	ino_t			current_inode;
 	blk_t			current_block;
 	dgrp_t			current_group;
-	int			inodes_left, blocks_left, groups_left;
-	int			inode_buffer_blocks;
+	ino_t			inodes_left;
+	blk_t			blocks_left;
+	dgrp_t			groups_left;
+	blk_t			inode_buffer_blocks;
 	char *			inode_buffer;
 	int			inode_size;
 	char *			ptr;
@@ -39,7 +47,7 @@ struct ext2_struct_inode_scan {
 	errcode_t		(*done_group)(ext2_filsys fs,
 					      ext2_inode_scan scan,
 					      dgrp_t group,
-					      void * private);
+					      void * priv_data);
 	void *			done_group_data;
 	int			bad_block_ptr;
 	int			scan_flags;
@@ -48,27 +56,33 @@ struct ext2_struct_inode_scan {
 
 static errcode_t create_icache(ext2_filsys fs)
 {
-	int	i;
+	errcode_t	retval;
+	int		i;
 	
 	if (fs->icache)
 		return 0;
-	fs->icache = malloc(sizeof(struct ext2_inode_cache));
+	retval = ext2fs_get_mem(sizeof(struct ext2_inode_cache), 
+				(void **) &fs->icache);
+	if (retval)
+		return retval;
+
 	memset(fs->icache, 0, sizeof(struct ext2_inode_cache));
-	fs->icache->buffer = malloc(fs->blocksize);
-	if (!fs->icache->buffer) {
-		free(fs->icache);
-		return ENOMEM;
+	retval = ext2fs_get_mem(fs->blocksize, (void **) &fs->icache->buffer);
+	if (retval) {
+		ext2fs_free_mem((void **) &fs->icache);
+		return retval;
 	}
 	fs->icache->buffer_blk = 0;
 	fs->icache->cache_last = -1;
 	fs->icache->cache_size = 4;
 	fs->icache->refcount = 1;
-	fs->icache->cache = malloc(sizeof(struct ext2_inode_cache_ent)
-				   * fs->icache->cache_size);
-	if (!fs->icache->cache) {
-		free(fs->icache->buffer);
-		free(fs->icache);
-		return ENOMEM;
+	retval = ext2fs_get_mem(sizeof(struct ext2_inode_cache_ent)
+				* fs->icache->cache_size,
+				(void **) &fs->icache->cache);
+	if (retval) {
+		ext2fs_free_mem((void **) &fs->icache->buffer);
+		ext2fs_free_mem((void **) &fs->icache);
+		return retval;
 	}
 	for (i=0; i < fs->icache->cache_size; i++)
 		fs->icache->cache[i].ino = 0;
@@ -80,7 +94,7 @@ errcode_t ext2fs_open_inode_scan(ext2_filsys fs, int buffer_blocks,
 {
 	ext2_inode_scan	scan;
 	errcode_t	retval;
-	errcode_t (*save_get_blocks)(ext2_filsys fs, ino_t ino, blk_t *blocks);
+	errcode_t (*save_get_blocks)(ext2_filsys f, ino_t ino, blk_t *blocks);
 
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
@@ -103,31 +117,39 @@ errcode_t ext2fs_open_inode_scan(ext2_filsys fs, int buffer_blocks,
 		fs->get_blocks = save_get_blocks;
 	}
 
-	scan = (ext2_inode_scan) malloc(sizeof(struct ext2_struct_inode_scan));
-	if (!scan)
-		return ENOMEM;
+	retval = ext2fs_get_mem(sizeof(struct ext2_struct_inode_scan),
+				(void **) &scan);
+	if (retval)
+		return retval;
 	memset(scan, 0, sizeof(struct ext2_struct_inode_scan));
 
 	scan->magic = EXT2_ET_MAGIC_INODE_SCAN;
 	scan->fs = fs;
 	scan->inode_size = EXT2_INODE_SIZE(fs->super);
 	scan->bytes_left = 0;
-	scan->current_group = -1;
+	scan->current_group = 0;
+	scan->groups_left = fs->group_desc_count - 1;
 	scan->inode_buffer_blocks = buffer_blocks ? buffer_blocks : 8;
-	scan->groups_left = fs->group_desc_count;
-	scan->inode_buffer = malloc(scan->inode_buffer_blocks * fs->blocksize);
+	scan->current_block = scan->fs->
+		group_desc[scan->current_group].bg_inode_table;
+	scan->inodes_left = EXT2_INODES_PER_GROUP(scan->fs->super);
+	scan->blocks_left = scan->fs->inode_blocks_per_group;
+	retval = ext2fs_get_mem((size_t) (scan->inode_buffer_blocks * 
+					  fs->blocksize),
+				(void **) &scan->inode_buffer);
 	scan->done_group = 0;
 	scan->done_group_data = 0;
 	scan->bad_block_ptr = 0;
-	if (!scan->inode_buffer) {
-		free(scan);
-		return ENOMEM;
+	if (retval) {
+		ext2fs_free_mem((void **) &scan);
+		return retval;
 	}
-	scan->temp_buffer = malloc(scan->inode_size);
-	if (!scan->temp_buffer) {
-		free(scan->inode_buffer);
-		free(scan);
-		return ENOMEM;
+	retval = ext2fs_get_mem(scan->inode_size,
+				(void **) &scan->temp_buffer);
+	if (retval) {
+		ext2fs_free_mem((void **) &scan->inode_buffer);
+		ext2fs_free_mem((void **) &scan);
+		return retval;
 	}
 	if (scan->fs->badblocks && scan->fs->badblocks->num)
 		scan->scan_flags |= EXT2_SF_CHK_BADBLOCKS;
@@ -140,11 +162,11 @@ void ext2fs_close_inode_scan(ext2_inode_scan scan)
 	if (!scan || (scan->magic != EXT2_ET_MAGIC_INODE_SCAN))
 		return;
 	
-	free(scan->inode_buffer);
+	ext2fs_free_mem((void **) &scan->inode_buffer);
 	scan->inode_buffer = NULL;
-	free(scan->temp_buffer);
+	ext2fs_free_mem((void **) &scan->temp_buffer);
 	scan->temp_buffer = NULL;
-	free(scan);
+	ext2fs_free_mem((void **) &scan);
 	return;
 }
 
@@ -152,7 +174,7 @@ void ext2fs_set_inode_callback(ext2_inode_scan scan,
 			       errcode_t (*done_group)(ext2_filsys fs,
 						       ext2_inode_scan scan,
 						       dgrp_t group,
-						       void * private),
+						       void * priv_data),
 			       void *done_group_data)
 {
 	if (!scan || (scan->magic != EXT2_ET_MAGIC_INODE_SCAN))
@@ -188,6 +210,9 @@ static errcode_t get_next_blockgroup(ext2_inode_scan scan)
 	scan->current_block = scan->fs->
 		group_desc[scan->current_group].bg_inode_table;
 
+	scan->current_inode = scan->current_group *
+		EXT2_INODES_PER_GROUP(scan->fs->super);
+
 	scan->bytes_left = 0;
 	scan->inodes_left = EXT2_INODES_PER_GROUP(scan->fs->super);
 	scan->blocks_left = scan->fs->inode_blocks_per_group;
@@ -210,7 +235,7 @@ errcode_t ext2fs_inode_scan_goto_blockgroup(ext2_inode_scan scan,
  * increasing order.
  */
 static errcode_t check_for_inode_bad_blocks(ext2_inode_scan scan,
-					    int *num_blocks)
+					    blk_t *num_blocks)
 {
 	blk_t	blk = scan->current_block;
 	badblocks_list	bb = scan->fs->badblocks;
@@ -258,7 +283,7 @@ static errcode_t check_for_inode_bad_blocks(ext2_inode_scan scan,
 	 * will be the bad block, which is handled in the above case.)
 	 */
 	if ((blk + *num_blocks) > bb->list[scan->bad_block_ptr])
-		*num_blocks = bb->list[scan->bad_block_ptr] - blk;
+		*num_blocks = (int) (bb->list[scan->bad_block_ptr] - blk);
 
 	return 0;
 }
@@ -269,7 +294,7 @@ static errcode_t check_for_inode_bad_blocks(ext2_inode_scan scan,
  */
 static errcode_t get_next_blocks(ext2_inode_scan scan)
 {
-	int		num_blocks;
+	blk_t		num_blocks;
 	errcode_t	retval;
 
 	/*
@@ -303,11 +328,12 @@ static errcode_t get_next_blocks(ext2_inode_scan scan)
 	if ((scan->scan_flags & EXT2_SF_BAD_INODE_BLK) ||
 	    (scan->current_block == 0)) {
 		memset(scan->inode_buffer, 0,
-		       num_blocks * scan->fs->blocksize);
+		       (size_t) num_blocks * scan->fs->blocksize);
 	} else {
 		retval = io_channel_read_blk(scan->fs->io,
 					     scan->current_block,
-					     num_blocks, scan->inode_buffer);
+					     (int) num_blocks,
+					     scan->inode_buffer);
 		if (retval)
 			return EXT2_ET_NEXT_INODE_READ;
 	}
@@ -320,6 +346,27 @@ static errcode_t get_next_blocks(ext2_inode_scan scan)
 	return 0;
 }
 
+#if 0
+/*
+ * Returns 1 if the entire inode_buffer has a non-zero size and
+ * contains all zeros.  (Not just deleted inodes, since that means
+ * that part of the inode table was used at one point; we want all
+ * zeros, which means that the inode table is pristine.)
+ */
+static inline int is_empty_scan(ext2_inode_scan scan)
+{
+	int	i;
+	
+	if (scan->bytes_left == 0)
+		return 0;
+
+	for (i=0; i < scan->bytes_left; i++)
+		if (scan->ptr[i])
+			return 0;
+	return 1;
+}
+#endif
+
 errcode_t ext2fs_get_next_inode(ext2_inode_scan scan, ino_t *ino,
 				struct ext2_inode *inode)
 {
@@ -332,7 +379,7 @@ errcode_t ext2fs_get_next_inode(ext2_inode_scan scan, ino_t *ino,
 	 * Do we need to start reading a new block group?
 	 */
 	if (scan->inodes_left <= 0) {
-	retry:
+	force_new_group:
 		if (scan->done_group) {
 			retval = (scan->done_group)
 				(scan->fs, scan, scan->current_group,
@@ -347,13 +394,18 @@ errcode_t ext2fs_get_next_inode(ext2_inode_scan scan, ino_t *ino,
 		retval = get_next_blockgroup(scan);
 		if (retval)
 			return retval;
-		if (scan->current_block == 0) {
-			if (scan->scan_flags & EXT2_SF_SKIP_MISSING_ITABLE) {
-				goto retry;
-			} else
-				return EXT2_ET_MISSING_INODE_TABLE;
-		}
 	}
+	/*
+	 * This is done outside the above if statement so that the
+	 * check can be done for block group #0.
+	 */
+	if (scan->current_block == 0) {
+		if (scan->scan_flags & EXT2_SF_SKIP_MISSING_ITABLE) {
+			goto force_new_group;
+		} else
+			return EXT2_ET_MISSING_INODE_TABLE;
+	}
+	
 
 	/*
 	 * Have we run out of space in the inode buffer?  If so, we
@@ -366,6 +418,14 @@ errcode_t ext2fs_get_next_inode(ext2_inode_scan scan, ino_t *ino,
 		retval = get_next_blocks(scan);
 		if (retval)
 			return retval;
+#if 0
+		/*
+		 * XXX test  Need check for used inode somehow.
+		 * (Note: this is hard.)
+		 */
+		if (is_empty_scan(scan))
+			goto force_new_group;
+#endif
 	}
 
 	retval = 0;
@@ -435,15 +495,15 @@ errcode_t ext2fs_read_inode (ext2_filsys fs, ino_t ino,
 			return 0;
 		}
 	}
-	if (ino > fs->super->s_inodes_count)
+	if ((ino == 0) || (ino > fs->super->s_inodes_count))
 		return EXT2_ET_BAD_INODE_NUM;
 	group = (ino - 1) / EXT2_INODES_PER_GROUP(fs->super);
 	offset = ((ino - 1) % EXT2_INODES_PER_GROUP(fs->super)) *
 		EXT2_INODE_SIZE(fs->super);
 	block = offset >> EXT2_BLOCK_SIZE_BITS(fs->super);
-	if (!fs->group_desc[group].bg_inode_table)
+	if (!fs->group_desc[(unsigned)group].bg_inode_table)
 		return EXT2_ET_MISSING_INODE_TABLE;
-	block_nr = fs->group_desc[group].bg_inode_table + block;
+	block_nr = fs->group_desc[(unsigned)group].bg_inode_table + block;
 	if (block_nr != fs->icache->buffer_blk) {
 		retval = io_channel_read_blk(fs->io, block_nr, 1,
 					     fs->icache->buffer);
@@ -452,7 +512,7 @@ errcode_t ext2fs_read_inode (ext2_filsys fs, ino_t ino,
 		fs->icache->buffer_blk = block_nr;
 	}
 	offset &= (EXT2_BLOCK_SIZE(fs->super) - 1);
-	ptr = ((char *) fs->icache->buffer) + offset;
+	ptr = ((char *) fs->icache->buffer) + (unsigned) offset;
 
 	memset(inode, 0, sizeof(struct ext2_inode));
 	length = EXT2_INODE_SIZE(fs->super);
@@ -460,7 +520,7 @@ errcode_t ext2fs_read_inode (ext2_filsys fs, ino_t ino,
 		length = sizeof(struct ext2_inode);
 	
 	if ((offset + length) > EXT2_BLOCK_SIZE(fs->super)) {
-		clen = EXT2_BLOCK_SIZE(fs->super) - offset;
+		clen = (int) (EXT2_BLOCK_SIZE(fs->super) - offset);
 		memcpy((char *) inode, ptr, clen);
 		length -= clen;
 		
@@ -525,7 +585,7 @@ errcode_t ext2fs_write_inode(ext2_filsys fs, ino_t ino,
 	if (!(fs->flags & EXT2_FLAG_RW))
 		return EXT2_ET_RO_FILSYS;
 
-	if (ino > fs->super->s_inodes_count)
+	if ((ino == 0) || (ino > fs->super->s_inodes_count))
 		return EXT2_ET_BAD_INODE_NUM;
 
 	if ((fs->flags & EXT2_FLAG_SWAP_BYTES) ||
@@ -538,11 +598,11 @@ errcode_t ext2fs_write_inode(ext2_filsys fs, ino_t ino,
 	offset = ((ino - 1) % EXT2_INODES_PER_GROUP(fs->super)) *
 		EXT2_INODE_SIZE(fs->super);
 	block = offset >> EXT2_BLOCK_SIZE_BITS(fs->super);
-	if (!fs->group_desc[group].bg_inode_table)
+	if (!fs->group_desc[(unsigned) group].bg_inode_table)
 		return EXT2_ET_MISSING_INODE_TABLE;
-	block_nr = fs->group_desc[group].bg_inode_table + block;
+	block_nr = fs->group_desc[(unsigned) group].bg_inode_table + block;
 	offset &= (EXT2_BLOCK_SIZE(fs->super) - 1);
-	ptr = (char *) fs->icache->buffer + offset;
+	ptr = (char *) fs->icache->buffer + (unsigned) offset;
 
 	length = EXT2_INODE_SIZE(fs->super);
 	clen = length;
@@ -558,7 +618,7 @@ errcode_t ext2fs_write_inode(ext2_filsys fs, ino_t ino,
 	}
 	
 	if ((offset + length) > EXT2_BLOCK_SIZE(fs->super)) {
-		clen = EXT2_BLOCK_SIZE(fs->super) - offset;
+		clen = (int) (EXT2_BLOCK_SIZE(fs->super) - offset);
 		length -= clen;
 	} else {
 		length = 0;
@@ -622,13 +682,16 @@ errcode_t ext2fs_check_directory(ext2_filsys fs, ino_t ino)
 	if (ino > fs->super->s_inodes_count)
 		return EXT2_ET_BAD_INODE_NUM;
 
-	if (fs->check_directory)
-		return (fs->check_directory)(fs, ino);
+	if (fs->check_directory) {
+		retval = (fs->check_directory)(fs, ino);
+		if (retval != EXT2_ET_CALLBACK_NOTHANDLED)
+			return retval;
+	}
 	retval = ext2fs_read_inode(fs, ino, &inode);
 	if (retval)
 		return retval;
 	if (!LINUX_S_ISDIR(inode.i_mode))
-		return ENOTDIR;
+		return EXT2_ET_NO_DIRECTORY;
 	return 0;
 }
 
