@@ -11,13 +11,15 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
-#if HAVE_ERRNO_H
-#include <errno.h>
 #endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -28,17 +30,20 @@ struct lookup_struct  {
 	int		found;
 };	
 
+#ifdef __TURBOC__
+#pragma argsused
+#endif
 static int lookup_proc(struct ext2_dir_entry *dirent,
 		       int	offset,
 		       int	blocksize,
 		       char	*buf,
-		       void	*private)
+		       void	*priv_data)
 {
-	struct lookup_struct *ls = (struct lookup_struct *) private;
+	struct lookup_struct *ls = (struct lookup_struct *) priv_data;
 
-	if (ls->len != dirent->name_len)
+	if (ls->len != (dirent->name_len & 0xFF))
 		return 0;
-	if (strncmp(ls->name, dirent->name, dirent->name_len))
+	if (strncmp(ls->name, dirent->name, (dirent->name_len & 0xFF)))
 		return 0;
 	*ls->inode = dirent->inode;
 	ls->found++;
@@ -63,7 +68,7 @@ errcode_t ext2fs_lookup(ext2_filsys fs, ino_t dir, const char *name,
 	if (retval)
 		return retval;
 
-	return (ls.found) ? 0 : ENOENT;
+	return (ls.found) ? 0 : EXT2_ET_FILE_NOT_FOUND;
 }
 
 
