@@ -23,6 +23,10 @@
 
 #include "ext2fs.h"
 
+#ifndef EXT2_FT_DIR
+#define EXT2_FT_DIR		2
+#endif
+
 /*
  * Create new directory block
  */
@@ -33,6 +37,8 @@ errcode_t ext2fs_new_dir_block(ext2_filsys fs, ino_t dir_ino, ino_t parent_ino,
 	errcode_t		retval;
 	char			*buf;
 	int			rec_len;
+	int			filetype = 0;
+	struct ext2fs_sb	*sb;
 
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
@@ -44,14 +50,17 @@ errcode_t ext2fs_new_dir_block(ext2_filsys fs, ino_t dir_ino, ino_t parent_ino,
 	dir->rec_len = fs->blocksize;
 
 	if (dir_ino) {
+		sb = (struct ext2fs_sb *) fs->super;
+		if (sb->s_feature_incompat & EXT2_FEATURE_INCOMPAT_FILETYPE)
+			filetype = EXT2_FT_DIR << 8;
 		/*
 		 * Set up entry for '.'
 		 */
 		dir->inode = dir_ino;
-		dir->name_len = 1;
+		dir->name_len = 1 | filetype;
 		dir->name[0] = '.';
-		rec_len = dir->rec_len - EXT2_DIR_REC_LEN(dir->name_len);
-		dir->rec_len = EXT2_DIR_REC_LEN(dir->name_len);
+		rec_len = dir->rec_len - EXT2_DIR_REC_LEN(1);
+		dir->rec_len = EXT2_DIR_REC_LEN(1);
 
 		/*
 		 * Set up entry for '..'
@@ -59,7 +68,7 @@ errcode_t ext2fs_new_dir_block(ext2_filsys fs, ino_t dir_ino, ino_t parent_ino,
 		dir = (struct ext2_dir_entry *) (buf + dir->rec_len);
 		dir->rec_len = rec_len;
 		dir->inode = parent_ino;
-		dir->name_len = 2;
+		dir->name_len = 2 | filetype;
 		dir->name[0] = '.';
 		dir->name[1] = '.';
 		
