@@ -100,7 +100,11 @@ void ext2fs_swap_inode(ext2_filsys fs, struct ext2_inode *t,
 		for (i = 0; i < EXT2_N_BLOCKS; i++)
 			t->i_block[i] = f->i_block[i];
 	}
+#if defined(HAVE_EXT2_INODE_VERSION)
 	t->i_version = ext2fs_swab32(f->i_version);
+#else
+	t->i_generation = ext2fs_swab32(f->i_generation);
+#endif
 	t->i_file_acl = ext2fs_swab32(f->i_file_acl);
 	t->i_dir_acl = ext2fs_swab32(f->i_dir_acl);
 	t->i_faddr = ext2fs_swab32(f->i_faddr);
