@@ -11,10 +11,15 @@
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#include <stdlib.h>
+#endif
 
+#if EXT2_FLAT_INCLUDES
+#include "ext2_fs.h"
+#else
 #include <linux/ext2_fs.h>
+#endif
 
 #include "ext2fs.h"
 
@@ -26,17 +31,21 @@ struct link_struct  {
 	int		done;
 };	
 
+#ifdef __TURBOC__
+#pragma argsused
+#endif
 static int unlink_proc(struct ext2_dir_entry *dirent,
 		     int	offset,
 		     int	blocksize,
 		     char	*buf,
-		     void	*private)
+		     void	*priv_data)
 {
-	struct link_struct *ls = (struct link_struct *) private;
+	struct link_struct *ls = (struct link_struct *) priv_data;
 
-	if (ls->name && (dirent->name_len != ls->namelen))
+	if (ls->name && ((dirent->name_len & 0xFF) != ls->namelen))
 		return 0;
-	if (ls->name && strncmp(ls->name, dirent->name, dirent->name_len))
+	if (ls->name && strncmp(ls->name, dirent->name,
+				dirent->name_len & 0xFF))
 		return 0;
 	if (ls->inode && (dirent->inode != ls->inode))
 		return 0;
@@ -46,6 +55,9 @@ static int unlink_proc(struct ext2_dir_entry *dirent,
 	return DIRENT_ABORT|DIRENT_CHANGED;
 }
 
+#ifdef __TURBOC__
+#pragma argsused
+#endif
 errcode_t ext2fs_unlink(ext2_filsys fs, ino_t dir, const char *name, ino_t ino,
 			int flags)
 {
