@@ -1,15 +1,21 @@
 /*
  * Listener loop for subsystem library libss.a.
  *
- *	$Header: /mit/krb5/.cvsroot/src/util/ss/listen.c,v 1.1 1993/06/03 12:31:07 tytso Exp $
+ *	$Header: /usr/src/CVS-REPO/e2fsprogs/lib/ss/listen.c,v 1.15 1999/10/23 01:16:22 tytso Exp $
  *	$Locker:  $
  * 
  * Copyright 1987, 1988 by MIT Student Information Processing Board
  *
- * For copyright information, see copyright.h.
+ * Permission to use, copy, modify, and distribute this software and
+ * its documentation for any purpose is hereby granted, provided that
+ * the names of M.I.T. and the M.I.T. S.I.P.B. not be used in
+ * advertising or publicity pertaining to distribution of the software
+ * without specific, written prior permission.  M.I.T. and the
+ * M.I.T. S.I.P.B. make no representations about the suitability of
+ * this software for any purpose.  It is provided "as is" without
+ * express or implied warranty.
  */
 
-#include "copyright.h"
 #include "ss_internal.h"
 #include <stdio.h>
 #include <setjmp.h>
@@ -21,7 +27,7 @@
 
 #ifndef	lint
 static char const rcs_id[] =
-    "$Header: /mit/krb5/.cvsroot/src/util/ss/listen.c,v 1.1 1993/06/03 12:31:07 tytso Exp $";
+    "$Header: /usr/src/CVS-REPO/e2fsprogs/lib/ss/listen.c,v 1.15 1999/10/23 01:16:22 tytso Exp $";
 #endif
 
 typedef void sigret_t;
