@@ -13,14 +13,19 @@
 
 #include "uuidP.h"
 
+#define UUCMP(u1,u2) if (u1 != u2) return((u1 < u2) ? -1 : 1);
+
 int uuid_compare(uuid_t uu1, uuid_t uu2)
 {
-	unsigned char 	*cp1, *cp2;
-	int		i;
+	struct uuid	uuid1, uuid2;
 
-	for (i=0, cp1 = uu1, cp2 = uu2; i < 16; i++)
-		if (*cp1++ != *cp2++)
-			return 0;
-	return 1;
+	uuid_unpack(uu1, &uuid1);
+	uuid_unpack(uu2, &uuid2);
+
+	UUCMP(uuid1.time_low, uuid2.time_low);
+	UUCMP(uuid1.time_mid, uuid2.time_mid);
+	UUCMP(uuid1.time_hi_and_version, uuid2.time_hi_and_version);
+	UUCMP(uuid1.clock_seq, uuid2.clock_seq);
+	return memcmp(uuid1.node, uuid2.node, 6);
 }
 
