@@ -45,6 +45,7 @@
 #include "ext2fs/ext2_io.h"
 
 const char * program_name = "badblocks";
+const char * done_string = "done                        \n";
 
 int v_flag = 0;			/* verbose */
 int w_flag = 0;			/* do r/w test */
@@ -98,10 +99,9 @@ static long do_test (int dev, char * buffer, int try, unsigned long block_size,
 	got = read (dev, buffer, try * block_size);
 	if (got < 0)
 		got = 0;	
-	if (got & (block_size - 1))
+	if (got & 511)
 		fprintf (stderr,
-			 "Weird value (%ld) in do_test: probably bugs\n",
-			 got);
+			 "Weird value (%ld) in do_test\n", got);
 	got /= block_size;
 	return got;
 }
@@ -173,7 +173,7 @@ static void test_ro (int dev, unsigned long blocks_count,
 	num_blocks = 0;
 	alarm(0);
 	if (s_flag || v_flag > 1)
-		fprintf(stderr, "done               \n");
+		fprintf(stderr, done_string);
 	fflush (stderr);
 	free (blkbuf);
 }
@@ -227,7 +227,7 @@ static void test_rw (int dev, unsigned long blocks_count,
 		num_blocks = 0;
 		alarm (0);
 		if (s_flag | v_flag)
-			fprintf(stderr, "done               \n");
+			fprintf(stderr, done_string);
 		flush_bufs (dev, 1);
 		if (s_flag | v_flag)
 			fprintf (stderr, "Reading and comparing: ");
@@ -255,7 +255,7 @@ static void test_rw (int dev, unsigned long blocks_count,
 		num_blocks = 0;
 		alarm (0);
 		if (s_flag | v_flag)
-			fprintf(stderr, "done           \n");
+			fprintf(stderr, done_string);
 		flush_bufs (dev, 0);
 	}
 }
