@@ -32,7 +32,9 @@
 #include <sys/stat.h>
 #include <limits.h>
 #include <stdio.h>
+#include <ctype.h>
 #include <string.h>
+#include <time.h>
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -131,6 +133,7 @@ int like_mount = 0;
 int notitle = 0;
 int parallel_root = 0;
 int progress = 0;
+int force_all_parallel = 0;
 char *progname;
 char *fstype = NULL;
 struct fs_info *filesys_info;
@@ -202,14 +205,14 @@ static void free_instance(struct fsck_instance *i)
 	return;
 }
 
-int parse_fstab_line(char *line, struct fs_info **ret_fs)
+static int parse_fstab_line(char *line, struct fs_info **ret_fs)
 {
 	char	*device, *mntpnt, *type, *opts, *freq, *passno, *cp;
 	struct fs_info *fs;
 
 	*ret_fs = 0;
 	strip_line(line);
-	if (cp = strchr(line, '#'))
+	if ((cp = strchr(line, '#')))
 		*cp = 0;	/* Ignore everything after the comment char */
 	cp = line;
 
@@ -264,7 +267,7 @@ static char *interpret_device(char *spec)
 /*
  * Load the filesystem database from /etc/fstab
  */
-static void load_fs_info(char *filename)
+static void load_fs_info(const char *filename)
 {
 	FILE	*f;
 	char	buf[1024];
@@ -356,7 +359,7 @@ static char *find_fsck(char *type)
   return(s ? prog : NULL);
 }
 
-static int progress_active()
+static int progress_active(NOARGS)
 {
 	struct fsck_instance *inst;
 
@@ -373,11 +376,12 @@ static int progress_active()
  * Execute a particular fsck program, and link it into the list of
  * child processes we are waiting for.
  */
-static int execute(char *type, char *device, char *mntpt, int interactive)
+static int execute(const char *type, char *device, char *mntpt,
+		   int interactive)
 {
 	char *s, *argv[80], prog[80];
 	int  argc, i;
-	struct fsck_instance *inst;
+	struct fsck_instance *inst, *p;
 	pid_t	pid;
 
 	inst = malloc(sizeof(struct fsck_instance));
@@ -394,7 +398,7 @@ static int execute(char *type, char *device, char *mntpt, int interactive)
 
 	if (progress & !progress_active()) {
 		if (strcmp(type, "ext2") == 0) {
-			argv[argc++] = "-C0";
+			argv[argc++] = string_copy("-C0");
 			inst->flags |= FLAG_PROGRESS;
 		}
 	}
@@ -429,12 +433,25 @@ static int execute(char *type, char *device, char *mntpt, int interactive)
 		exit(EXIT_ERROR);
 	}
 
+	for (i=0; i < argc; i++)
+		free(argv[i]);
+	
 	inst->pid = pid;
 	inst->prog = string_copy(prog);
 	inst->type = string_copy(type);
 	inst->device = string_copy(device);
-	inst->next = instance_list;
-	instance_list = inst;
+	inst->start_time = time(0);
+	inst->next = NULL;
+
+	/*
+	 * Find the end of the list, so we add the instance on at the end.
+	 */
+	for (p = instance_list; p && p->next; p = p->next);
+
+	if (p)
+		p->next = inst;
+	else
+		instance_list = inst;
 	
 	return 0;
 }
@@ -460,31 +477,34 @@ static struct fsck_instance *wait_one(NOARGS)
 		return(inst);
 	}
 
-retry:
-	pid = wait(&status);
-	if (pid < 0) {
-		if ((errno == EINTR) || (errno == EAGAIN))
-			goto retry;
-		if (errno == ECHILD) {
-			fprintf(stderr,
-				"%s: wait: No more child process?!?\n",
-				progname);
-			return NULL;
+	/*
+	 * gcc -Wall fails saving throw against stupidity
+	 * (inst and prev are thought to be uninitialized variables)
+	 */
+	inst = prev = NULL;
+	
+	do {
+		pid = wait(&status);
+		if (pid < 0) {
+			if ((errno == EINTR) || (errno == EAGAIN))
+				continue;
+			if (errno == ECHILD) {
+				fprintf(stderr,
+					"%s: wait: No more child process?!?\n",
+					progname);
+				return NULL;
+			}
+			perror("wait");
+			continue;
 		}
-		perror("wait");
-		goto retry;
-	}
-	for (prev = 0, inst = instance_list;
-	     inst;
-	     prev = inst, inst = inst->next) {
-		if (inst->pid == pid)
-			break;
-	}
-	if (!inst) {
-		printf("Unexpected child process %d, status = 0x%x\n",
-		       pid, status);
-		goto retry;
-	}
+		for (prev = 0, inst = instance_list;
+		     inst;
+		     prev = inst, inst = inst->next) {
+			if (inst->pid == pid)
+				break;
+		}
+	} while (!inst);
+
 	if (WIFEXITED(status)) 
 		status = WEXITSTATUS(status);
 	else if (WIFSIGNALED(status)) {
@@ -514,7 +534,19 @@ retry:
 				continue;
 			if (strcmp(inst2->type, "ext2"))
 				continue;
-			kill(inst2->pid, SIGUSR1);
+			/*
+			 * If we've just started the fsck, wait a tiny
+			 * bit before sending the kill, to give it
+			 * time to set up the signal handler
+			 */
+			if (inst2->start_time < time(0)+2) {
+				if (fork() == 0) {
+					sleep(1);
+					kill(inst2->pid, SIGUSR1);
+					exit(0);
+				}
+			} else
+				kill(inst2->pid, SIGUSR1);
 			break;
 		}
 	}
@@ -552,7 +584,7 @@ static int wait_all(NOARGS)
  */
 static void fsck_device(char *device, int interactive)
 {
-	char	*type = 0;
+	const char *type = 0;
 	struct fs_info *fsent;
 	int retval;
 
@@ -668,6 +700,9 @@ static int device_already_active(char *device)
 	struct fsck_instance *inst;
 	const char *base = base_device(device);
 
+	if (force_all_parallel)
+		return 0;
+
 	for (inst = instance_list; inst; inst = inst->next) {
 		if (!strcmp(base, base_device(inst->device)))
 			return 1;
@@ -883,6 +918,8 @@ static void PRS(int argc, char *argv[])
 			opt = 0;
 		}
 	}
+	if (getenv("FSCK_FORCE_ALL_PARALLEL"))
+		force_all_parallel++;
 }
 
 int main(int argc, char *argv[])
@@ -891,7 +928,7 @@ int main(int argc, char *argv[])
 	int status = 0;
 	int interactive = 0;
 	char *oldpath = getenv("PATH");
-	char *fstab;
+	const char *fstab;
 
 	PRS(argc, argv);
 
@@ -915,13 +952,13 @@ int main(int argc, char *argv[])
 		fsck_path = string_copy(fsck_prefix_path);
 	}
 	
+	if ((num_devices == 1) || (serialize))
+		interactive = 1;
+
 	/* If -A was specified ("check all"), do that! */
 	if (doall)
 		return check_all();
 
-	if ((num_devices == 1) || (serialize))
-		interactive = 1;
-
 	for (i = 0 ; i < num_devices; i++) {
 		fsck_device(devices[i], interactive);
 		if (serialize) {
