@@ -14,6 +14,7 @@
  * 93/10/30	- Creation
  * 93/11/13	- Replace stat() calls by lstat() to avoid loops
  * 94/02/27	- Integrated in Ted's distribution
+ * 98/12/29	- Display version info only when -V specified (G M Sipe)
  */
 
 #include <sys/types.h>
@@ -47,12 +48,13 @@ int all = 0;
 int d_opt = 0;
 int l_opt = 0;
 int recursive = 0;
+int verbose = 0;
 int v_opt = 0;
 
-static void volatile usage (void)
+static void usage(void)
 {
-	fprintf (stderr, "Usage: %s [-Radlv] [files...]\n", program_name);
-	exit (1);
+	fprintf(stderr, "Usage: %s [-RVadlv] [files...]\n", program_name);
+	exit(1);
 }
 
 static void list_attributes (const char * name)
@@ -119,22 +121,22 @@ static int lsattr_dir_proc (const char * dir_name, struct dirent * de, void * pr
 	return 0;
 }
 
-void main (int argc, char ** argv)
+int main (int argc, char ** argv)
 {
-	char c;
+	int c;
 	int i;
 
-	fprintf (stderr, "lsattr %s, %s for EXT2 FS %s, %s\n",
-		 E2FSPROGS_VERSION, E2FSPROGS_DATE,
-		 EXT2FS_VERSION, EXT2FS_DATE);
 	if (argc && *argv)
 		program_name = *argv;
-	while ((c = getopt (argc, argv, "Radlv")) != EOF)
+	while ((c = getopt (argc, argv, "RVadlv")) != EOF)
 		switch (c)
 		{
 			case 'R':
 				recursive = 1;
 				break;
+			case 'V':
+				verbose = 1;
+				break;
 			case 'a':
 				all = 1;
 				break;
@@ -148,12 +150,17 @@ void main (int argc, char ** argv)
 				v_opt = 1;
 				break;
 			default:
-				usage ();
+				usage();
 		}
 
+	if (verbose)
+		fprintf (stderr, "lsattr %s, %s for EXT2 FS %s, %s\n",
+			 E2FSPROGS_VERSION, E2FSPROGS_DATE,
+			 EXT2FS_VERSION, EXT2FS_DATE);
 	if (optind > argc - 1)
 		lsattr_args (".");
 	else
 		for (i = optind; i < argc; i++)
 			lsattr_args (argv[i]);
+	exit(0);
 }
