@@ -19,8 +19,10 @@
 #include <string.h>
 #include <fcntl.h>
 #include <ctype.h>
-#include <termios.h>
 #include <time.h>
+#ifdef linux
+#include <sys/utsname.h>
+#endif
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -38,7 +40,6 @@
 #endif
 #include <sys/ioctl.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 
 #ifdef HAVE_LINUX_FS_H
 #include <linux/fs.h>
@@ -46,10 +47,12 @@
 #include <linux/ext2_fs.h>
 #ifdef HAVE_LINUX_MAJOR_H
 #include <linux/major.h>
+#include <sys/stat.h>		/* Only need sys/stat.h for major nr test */
 #endif
 
 #include "et/com_err.h"
 #include "uuid/uuid.h"
+#include "e2p/e2p.h"
 #include "ext2fs/ext2fs.h"
 #include "../version.h"
 
@@ -91,8 +94,9 @@ static void usage(NOARGS)
 	"[-f fragment-size]\n\t[-i bytes-per-inode] "
 	" [-N number-of-inodes]\n\t[-m reserved-blocks-percentage] "
 	"[-o creator-os] [-g blocks-per-group]\n\t[-L volume-label] "
-	"[-M last-mounted-directory] [-r fs-revision]\n\t[-R raid_opts]"
-	"[-s sparse-super-flag] [-qvSV] device [blocks-count]\n",
+	"[-M last-mounted-directory] [-O feature[,...]]\n\t"
+	"[-r fs-revision] [-R raid_opts] [-s sparse-super-flag]\n\t"
+	"[-qvSV] device [blocks-count]\n", 
 		program_name);
 	exit(1);
 }
@@ -190,17 +194,17 @@ static void check_mount(NOARGS)
 /*
  * This function sets the default parameters for a filesystem
  *
- * The type is specified by the user.  The size is minimum size for
- * which a set of parameters applies, with a size of zero meaning that
- * it is the default parameter for the type.  Note that order is
- * important in the table below.
+ * The type is specified by the user.  The size is the maximum size
+ * (in megabytes) for which a set of parameters applies, with a size
+ * of zero meaning that it is the default parameter for the type.
+ * Note that order is important in the table below.
  */
 static char default_str[] = "default";
 struct mke2fs_defaults {
-	char	*type;
-	int	size;
-	int	blocksize;
-	int	inode_ratio;
+	const char	*type;
+	int		size;
+	int		blocksize;
+	int		inode_ratio;
 } settings[] = {
 	{ default_str, 0, 4096, 8192 },
 	{ default_str, 512, 1024, 4096 },
@@ -209,14 +213,14 @@ struct mke2fs_defaults {
 	{ 0, 0, 0, 0},
 };
 
-static void set_fs_defaults(char *fs_type, struct ext2fs_sb *param,
+static void set_fs_defaults(char *fs_type, struct ext2fs_sb *super,
 			    int blocksize, int *inode_ratio)
 {
 	int	megs;
 	int	ratio = 0;
 	struct mke2fs_defaults *p;
 
-	megs = (param->s_blocks_count * (EXT2_BLOCK_SIZE(param) / 1024) /
+	megs = (super->s_blocks_count * (EXT2_BLOCK_SIZE(super) / 1024) /
 		1024);
 	if (inode_ratio)
 		ratio = *inode_ratio;
@@ -232,12 +236,12 @@ static void set_fs_defaults(char *fs_type, struct ext2fs_sb *param,
 		if (ratio == 0)
 			*inode_ratio = p->inode_ratio;
 		if (blocksize == 0) {
-			param->s_log_frag_size = param->s_log_block_size =
+			super->s_log_frag_size = super->s_log_block_size =
 				log2(p->blocksize >> EXT2_MIN_BLOCK_LOG_SIZE);
 		}
 	}
 	if (blocksize == 0)
-		param->s_blocks_count /= EXT2_BLOCK_SIZE(param) / 1024;
+		super->s_blocks_count /= EXT2_BLOCK_SIZE(super) / 1024;
 }
 
 /*
@@ -563,7 +567,7 @@ static void show_stats(ext2_filsys fs)
 	printf("OS type: ");
 	switch (fs->super->s_creator_os) {
 	    case EXT2_OS_LINUX: printf ("Linux"); break;
-	    case EXT2_OS_HURD:  printf ("GNU/hurd");   break;
+	    case EXT2_OS_HURD:  printf ("GNU/Hurd");   break;
 	    case EXT2_OS_MASIX: printf ("Masix"); break;
 	    default:		printf ("(unknown os)");
         }
@@ -596,6 +600,8 @@ static void show_stats(ext2_filsys fs)
 		group_block += s->s_blocks_per_group;
 		if (!ext2fs_bg_has_super(fs, i))
 			continue;
+		if (i != 1)
+			printf(", ");
 		need = log10(group_block) + 2;
 		if (need > col_left) {
 			printf("\n\t");
@@ -603,8 +609,6 @@ static void show_stats(ext2_filsys fs)
 		}
 		col_left -= need;
 		printf("%u", group_block);
-		if (i != fs->group_desc_count - 1)
-			printf(", ");
 	}
 	printf("\n\n");
 }
@@ -697,26 +701,43 @@ static void parse_raid_opts(const char *opts)
 	}
 }	
 
+static __u32 ok_features[3] = {
+	0,					/* Compat */
+	EXT2_FEATURE_INCOMPAT_FILETYPE,		/* Incompat */
+	EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER	/* R/O compat */
+};
 
 
 static void PRS(int argc, char *argv[])
 {
-	int	c;
-	int	size;
-	char	* tmp;
-	blk_t	max = 8192;
-	int	blocksize = 0;
-	int	inode_ratio = 0;
-	int	reserved_ratio = 5;
-	ino_t	num_inodes = 0;
+	int		c;
+	int		size;
+	char *		tmp;
+	blk_t		max = 8192;
+	int		blocksize = 0;
+	int		inode_ratio = 0;
+	int		reserved_ratio = 5;
+	ino_t		num_inodes = 0;
 	errcode_t	retval;
-	int	sparse_option = 1;
-	char	*oldpath = getenv("PATH");
+	int		sparse_option = 0;
+	char *		oldpath = getenv("PATH");
 	struct ext2fs_sb *param_ext2 = (struct ext2fs_sb *) &param;
-	char	*raid_opts = 0;
-	char	*fs_type = 0;
-	blk_t	dev_size;
-	
+	char *		raid_opts = 0;
+	char *		fs_type = 0;
+	const char *	feature_set = "filetype,sparse_super";
+	blk_t		dev_size;
+#ifdef linux
+	struct 		utsname ut;
+
+	if (uname(&ut)) {
+		perror("uname");
+		exit(1);
+	}
+	if ((ut.release[0] == '1') ||
+	    (ut.release[0] == '2' && ut.release[1] == '.' &&
+	     ut.release[2] < '2' && ut.release[3] == '.'))
+		feature_set = 0;
+#endif
 	/* Update our PATH to include /sbin  */
 	if (oldpath) {
 		char *newpath;
@@ -741,7 +762,7 @@ static void PRS(int argc, char *argv[])
 	if (argc && *argv)
 		program_name = *argv;
 	while ((c = getopt (argc, argv,
-			    "b:cf:g:i:l:m:no:qr:R:s:tvI:ST:FL:M:N:V")) != EOF)
+		    "b:cf:g:i:l:m:no:qr:R:s:tvI:ST:FL:M:N:O:V")) != EOF)
 		switch (c) {
 		case 'b':
 			blocksize = strtoul(optarg, &tmp, 0);
@@ -845,6 +866,9 @@ static void PRS(int argc, char *argv[])
 		case 'M':
 			mount_dir = optarg;
 			break;
+		case 'O':
+			feature_set = optarg;
+			break;
 		case 'R':
 			raid_opts = optarg;
 			break;
@@ -928,7 +952,7 @@ static void PRS(int argc, char *argv[])
 	 * Calculate number of inodes based on the inode ratio
 	 */
 	param.s_inodes_count = num_inodes ? num_inodes : 
-		((long long) param.s_blocks_count * EXT2_BLOCK_SIZE(&param))
+		((__u64) param.s_blocks_count * EXT2_BLOCK_SIZE(&param))
 			/ inode_ratio;
 
 	/*
@@ -936,19 +960,20 @@ static void PRS(int argc, char *argv[])
 	 */
 	param.s_r_blocks_count = (param.s_blocks_count * reserved_ratio) / 100;
 
-	/*
-	 * If we are using revision #1, use the sparse super feature
-	 * by default
-	 */
 #ifdef EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER
-	if ((sparse_option == 1)
-#ifdef EXT2_DYNAMIC_REV
-	    || ((param.s_rev_level >= EXT2_DYNAMIC_REV) && (!sparse_option))
-#endif
-	    ) 
+	if (sparse_option)
 		param_ext2->s_feature_ro_compat |=
 			EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER;
 #endif
+	if (feature_set && !strncasecmp(feature_set, "none", 4))
+		feature_set = 0;
+	if (feature_set && e2p_edit_feature(feature_set,
+					    &param_ext2->s_feature_compat,
+					    ok_features)) {
+		fprintf(stderr, "Invalid filesystem option set: %s\n",
+			feature_set);
+		exit(1);
+	}
 }
 					
 int main (int argc, char *argv[])
