@@ -9,11 +9,8 @@ while (<>) {
     }
     next if (/^$/);	# skip blank lines
     $linelen = 0;
-    @words = split;
-    while (defined($word = shift @words)) {
-	$word =~ s#\$\(srcdir\)/\.\./version.h#\$\(top_srcdir\)/version.h#;
-	$word =~ s#\$\(srcdir\)/.\.\/\.\./version.h#\$\(top_srcdir\)/version.h#;
-	$word =~ s#\$\(srcdir\)/.\.\/et/com_err.h#\$\(top_srcdir\)/lib/et/com_err.h#;
+    split;
+    while (defined($word = shift @_)) {
 	if ($linelen > 0) {
 	    printf(" ");
 	}
